
/* Revision History
 *
 * $Id: functions.cc,v 1.8 2002/09/17 09:36:01 gvacanti Exp $
 *
 */


#include <dlfcn.h>


#include "Tracer.h"
#include "functions.h"
#include "table.h"
#include "Interpreter.h"
#include "param.h"
#include "stringutil.h"


StandardFunctions standardFunctions;


static Token
dump( const vector<Token> &	arguments )
{

	PROC

	TableT * lookupTablePtr = static_cast<TableT *>( arguments[ 0 ].asPointer() );

	cout << * lookupTablePtr;

	RETURN Token();
}


static Token
debug( const vector<Token> &	arguments )
{
	PROC
	for( unsigned int i = 1; i < arguments.size(); ++i )
	{
		cout << arguments[ i - 1 ] << endl;
	} // for
	RETURN Token();
}


static Token
print( const vector<Token> & arguments )
{
	PROC

	for( unsigned int i = 0; i < arguments.size() - 1; ++i )
	{
		Token token = arguments[ i ];

		switch( token.id() )
		{
			case Token::Integer :
				cout << token.asInt();
				break;
			case Token::Real :
				cout << token.asReal();
				break;
			case Token::String :
				cout << token.asString();
				break;
			case Token::Character :
				cout << token.asCharacter();
				break;
			case Token::Ptr :
				cout << token.asPointerDescriptor().raw();
				break;
			case Token::Pointer :
				cout << token.asPointer();
				break;
			default :
				cerr << "Unable to print value of given token" << endl;
				exit( 1 );
				break;
		} // switch
	} // for

	RETURN Token();
}


static void
filter( string & text, char removeThisCharacter )
{
	PROC
	for( unsigned int i = 0; i < text.length(); ++i )
	{
		if( text[ i ] == removeThisCharacter )
		{
			text[ i ] = ' ';
		} // if
	} // for
	ENDPROC
}


static Token
filterText( const vector<Token> &	arguments )
{

	PROC

	assert( arguments.size() > 0 );

	string		text = arguments[ 0 ].asString();

	for( unsigned int i = 1; i < arguments.size() - 1; ++i )
	{
		filter( text, arguments[ 1 ].asCharacter() );

	} // for
	RETURN text;
}


Token
param( const vector<Token> & arguments )
{
	PROC
	Token		result;

	string		name( arguments[ 0 ].asString() );

	//switch( (*gParameters)[ name ] -> getType() )
	//{
	//	case Bool :
	//		break;
	//	case
	//		break;
	//	default :
	//		break;
	//} // switch

	//RETURN stringParameter( name );
	RETURN unquote( (*gParameters)[ name ] -> asString() );
}


Token
dlOpen( const vector<Token>& arguments )
{
	PROC
	string		fileName( arguments[ 0 ].asString() );
	int		flag( arguments[ 1 ].asInt() );

	RETURN dlopen( fileName.c_str(), flag );

}



Token
dlClose( const vector<Token>& arguments )
{
	PROC
	const void *		handle( arguments[ 0 ].asPointer() );

	RETURN dlclose( const_cast<void *>( handle ));

}


Token
dlSym( const vector<Token>& arguments )
{
	PROC
	const void *		handle( arguments[ 0 ].asPointer() );
	string			symbol( arguments[ 1 ].asString() );

	RETURN dlsym( const_cast<void *>( handle ), symbol.c_str() );

}


Token
dlError( const vector<Token>& )
{
	PROC
	RETURN dlerror();

}


void
StandardFunctions::addFunctionsToTable( TableT & table ) const
{
	PROC


	struct Lookups lookups[] = {
	{ string("debug"),	Token( Functor( Functor::Internal, (void *)debug,	Token::Undefined)) },
	{ string("dump"),	Token( Functor( Functor::Internal, (void *)dump,	Token::Undefined)) },
	{ string("filterText"),	Token( Functor( Functor::Internal, (void *)filterText,	Token::Undefined)) },
	{ string( "param" ), 	Token( Functor( Functor::Internal, (void *)param,	Token::Undefined)) },
	{ string("print"),	Token( Functor( Functor::Internal, (void *)&print,	Token::Undefined)) },
	{ string("dlclose"),	Token( Functor( Functor::Internal, (void *)&dlClose, 	Token::Undefined)) },
	{ string("dlerror"),	Token( Functor( Functor::Internal, (void *)&dlError,	Token::Undefined)) },
	{ string("dlopen"),	Token( Functor( Functor::Internal, (void *)&dlOpen, 	Token::Undefined)) },
	{ string("dlsym"),	Token( Functor( Functor::Internal, (void *)&dlSym, 	Token::Undefined)) },
#ifndef DECOSF
	{ string("RTLD_GLOBAL"),Token( RTLD_GLOBAL ) },
#endif
	{ string("RTLD_LAZY"),	Token( RTLD_LAZY) },
	{ string("RTLD_NOW"),	Token( RTLD_NOW) }
};

	const unsigned int	numberOfFunctions =

					sizeof lookups /
					sizeof( Lookups );


	// Construct lookup table.
	for( unsigned int i = 0; i < numberOfFunctions; ++i )
	{
		table.Insert( lookups[ i ].name, lookups[ i ].token );

	} // for

	ENDPROC
}


